@@ -4,7 +4,7 @@ use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use Cassandra::Client::AsyncAnyEvent;
 use Cassandra::Client::AsyncEV;
@@ -16,7 +16,7 @@ use Cassandra::Client::Policy::Retry::Default;
 use Cassandra::Client::Policy::Retry;
 use Cassandra::Client::Policy::Throttle::Adaptive;
 use Cassandra::Client::Pool;
-use Cassandra::Client::Util qw/series/;
+use Cassandra::Client::Util qw/series whilst/;
 
 use Clone qw/clone/;
 use List::Util qw/shuffle/;
@@ -90,6 +90,7 @@ sub _connect {
         my $contact_point= shift @contact_points;
         if (!$contact_point) {
             delete $self->{connecting};
+            undef $next_connect;
             return _cb($_, "Unable to connect to any Cassandra server. Last error: $last_error") for @{delete $self->{connect_callbacks}};
         }
 
@@ -119,6 +120,7 @@ sub _connect {
                 return $next_connect->();
             }
 
+            undef $next_connect;
             $self->{connected}= 1;
             delete $self->{connecting};
             _cb($_) for @{delete $self->{connect_callbacks}};
@@ -363,25 +365,33 @@ sub _each_page {
     my $params_copy= $params ? clone($params) : undef;
     my $attribs_copy= $attribs ? clone($attribs) : undef;
 
-    my $next_page; $next_page= sub {
-        $self->_execute(sub {
-            # Completion handler, with page data (or an error)
-            my ($error, $result)= @_;
-            return _cb($callback, $error) if $error;
+    my $done= 0;
+    whilst(
+        sub { !$done },
+        sub {
+            my $next= shift;
 
-            my $next_page_id= $result->next_page;
-            _cb($page_callback, $result); # Note that page_callback doesn't get an error argument, that's intentional
-            if ($next_page_id) {
-                $attribs_copy->{page}= $next_page_id;
-                $next_page->();
-                return;
-            } else {
-                # Done!
-                return _cb($callback);
-            }
-        }, $query, $params_copy, $attribs_copy);
-    };
-    $next_page->();
+            $self->_execute(sub {
+                # Completion handler, with page data (or an error)
+                my ($error, $result)= @_;
+                return $next->($error) if $error;
+
+                my $next_page_id= $result->next_page;
+                _cb($page_callback, $result); # Note that page_callback doesn't get an error argument, that's intentional
+
+                if ($next_page_id) {
+                    $attribs_copy->{page}= $next_page_id;
+                } else {
+                    $done= 1;
+                }
+                return $next->();
+            }, $query, $params_copy, $attribs_copy);
+        },
+        sub {
+            my $error= shift;
+            return _cb($callback, $error);
+        }
+    );
 
     return;
 }
@@ -605,8 +615,8 @@ Executes a single query on Cassandra, and fetch the results (if any).
 For queries that have large amounts of result rows and end up spanning multiple pages, C<each_page> is the function you need. C<execute> does not handle pagination, and may end up missing rows unless pagination is implemented by its user through the C<page> attribute.
 
     $client->execute(
-        "UPDATE my_table SET column=? WHERE id=?",
-        [ 2, 5 ],
+        "UPDATE my_table SET column=:new_column WHERE id=:id",
+        { new_column => 2, id => 5 },
         { consistency => "quorum" },
     );
 
