@@ -7,20 +7,23 @@ sub new {
     my ($class, $config)= @_;
 
     my $self= bless {
-        anyevent            => 0,
-        contact_points      => undef,
-        port                => 9042,
-        cql_version         => undef,
-        keyspace            => undef,
-        compression         => undef,
-        default_consistency => undef,
-        max_page_size       => 5000,
-        max_connections     => 2,
-        timer_granularity   => 0.1,
-        request_timeout     => 11,
-        warmup              => 0,
-        throttler           => undef,
-        throttler_config    => undef,
+        anyevent                => 0,
+        contact_points          => undef,
+        port                    => 9042,
+        cql_version             => undef,
+        keyspace                => undef,
+        compression             => undef,
+        default_consistency     => undef,
+        max_page_size           => 5000,
+        max_connections         => 2,
+        timer_granularity       => 0.1,
+        request_timeout         => 11,
+        warmup                  => 0,
+        throttler               => undef,
+        throttler_config        => undef,
+        max_concurrent_queries  => 1000,
+        command_queue           => "Default",
+        command_queue_config    => undef,
     }, $class;
 
     if (my $cp= $config->{contact_points}) {
@@ -37,7 +40,7 @@ sub new {
     }
 
     # Numbers
-    for (qw/port timer_granularity request_timeout max_connections/) {
+    for (qw/port timer_granularity request_timeout max_connections max_concurrent_queries/) {
         if (defined($config->{$_})) {
             $self->{$_}= 0+ $config->{$_};
         }
@@ -56,7 +59,7 @@ sub new {
     }
 
     # Arbitrary hashes
-    for (qw/throttler_config/) {
+    for (qw/throttler_config command_queue_config/) {
         if (exists($config->{$_})) {
             $self->{$_}= $config->{$_};
         }
