@@ -431,35 +431,41 @@ sub wait_for_schema_agreement {
     my $wait_delay= 0.5;
     my $max_wait= 5;
 
-    my $wait_and_check; $wait_and_check= sub {
-        $waited += $wait_delay;
-        series([
-            sub {
-                my ($next)= @_;
-                $self->{async_io}->timer($next, $wait_delay);
-            },
-            sub {
-                my ($next)= @_;
-                $self->get_network_status($next);
-            },
-        ], sub {
-            my ($error, $network_status)= @_;
-            return $callback->($error) if $error;
+    my $done;
+    whilst(
+        sub { !$done },
+        sub {
+            my ($whilst_next)= @_;
 
-            my %versions;
-            $versions{$_->{schema_version}}= 1 for values %$network_status;
-            if (keys %versions > 1) {
-                if ($waited < $max_wait) {
-                    return $wait_and_check->();
+            series([
+                sub {
+                    my ($next)= @_;
+                    $self->{async_io}->timer($next, $wait_delay);
+                },
+                sub {
+                    my ($next)= @_;
+                    $waited += $wait_delay;
+                    $self->get_network_status($next);
+                },
+            ], sub {
+                my ($error, $network_status)= @_;
+                return $whilst_next->($error) if $error;
+
+                my %versions;
+                $versions{$_->{schema_version}}= 1 for values %$network_status;
+                if (keys %versions > 1) {
+                    if ($waited >= $max_wait) {
+                        return $whilst_next->("wait_for_schema_agreement timed out after $waited seconds");
+                    }
                 } else {
-                    return $callback->("wait_for_schema_agreement timed out after $waited seconds");
+                    $done= 1;
                 }
-            }
-            return $callback->();
-        });
-    };
+                return $whilst_next->();
+            });
+        },
+        $callback,
+    );
 
-    $wait_and_check->();
     return;
 }
 
@@ -670,6 +676,9 @@ sub setup_io {
     my ($self)= @_;
     $self->{async_io}->register($self->{fileno}, $self);
     $self->{async_io}->register_read($self->{fileno});
+
+    $self->{pending_write}= '';
+    $self->{async_io}->register_write($self->{fileno});
     return;
 }
 
