@@ -6,6 +6,8 @@ use warnings;
 use Cassandra::Client::Protocol qw/:constants BIGINT_SUPPORTED pack_long/;
 use Math::BigInt;
 use Encode;
+use POSIX qw/floor/;
+use Ref::Util qw/is_hashref/;
 
 use Exporter 'import';
 our @EXPORT_OK= qw/
@@ -37,13 +39,17 @@ my %types= (
     TYPE_LIST       ,=> [ \&e_list ],
     TYPE_MAP        ,=> [ \&e_map ],
     TYPE_SET        ,=> [ \&e_set ],
-
+    TYPE_TINYINT    ,=> [ 'c', 1 ],
+    TYPE_SMALLINT   ,=> [ 's>', 2 ],
+    TYPE_TIME       ,=> [ \&e_time ],
+    TYPE_DATE       ,=> [ \&e_date ],
 );
 
 sub make_encoder {
     my ($metadata)= @_;
 
     my @columns= @{$metadata->{columns}};
+    my @names= map { $_->[2] } @columns;
 
     my $code= <<EOC;
 
@@ -51,8 +57,13 @@ sub make_encoder {
     my \$length= pack('n', @{[ 0+@columns ]});
     my \$true= pack('l>c', 1, 1);
     my \$false= pack('l>c', 1, 0);
+
     sub {
-        local *INPUT= \$_[0];
+        my \$input_ref= \$_[0];
+        if (is_hashref(\$input_ref)) {
+            \$input_ref= [ @\$input_ref{\@names} ];
+        }
+        local *INPUT= \$input_ref;
         my \$row= \$length;
 
 @{[ map { my $i= $_; my $column= $columns[$i]; <<EOP } 0..$#columns
@@ -240,4 +251,49 @@ $output .= pack('l>/a', pack('l>', \$scale). substr(\$output_with_length_bytes,
 EOC
 }
 
+sub e_time {
+    my ($type, $input, $output)= @_;
+
+    my $regex= qr/\A(\d+)(?::(\d+)(?::(\d+)(?:\.(\d+))?)?)?\z/;
+    return <<EOC;
+{
+    if ($input =~ m#$regex#) {
+        my (\$hours, \$minutes, \$seconds, \$ns)= (\$1, \$2, \$3, \$4);
+        my \$timeval= ((((\$hours||0) * 3600) + ((\$minutes||0) * 60) + (\$seconds||0))%86400) . substr((\$ns||0)."000000000", 0, 9);
+        $output .= pack('l>', 8) . ( BIGINT_SUPPORTED ? pack('q>', \$timeval) : pack_long(\$timeval) );
+    } else {
+        my \$value= $input;
+        warn "Invalid value for TIME type: \$value";
+        $output .= \$null;
+    }
+}
+EOC
+}
+
+sub e_date {
+    my ($type, $input, $output)= @_;
+
+    my $regex= qr/\A(-?\d+)-(\d+)-(\d+)\z/;
+    return <<EOC;
+{
+    if ($input =~ m#$regex#) {
+        my (\$year, \$month, \$day)= (\$1, \$2, \$3);
+
+        my \$v_a= ((\$month == 1 || \$month == 2) ? 1 : 0);
+        my \$y= \$year + 4800 - \$v_a;
+        my \$m= \$month + (12 * \$v_a) - 3;
+        my \$jdn= \$day + floor(((153 * \$m) + 2) / 5) + (365 * \$y) + floor(\$y / 4) - floor(\$y / 100) + floor(\$y / 400);
+
+        my \$days= (1 << 31) - 2472633 + \$jdn;
+
+        $output .= pack('l>L>', 4, \$days);
+    } else {
+        my \$value= $input;
+        warn "Invalid value for DATE type: \$value";
+        $output .= \$null;
+    }
+}
+EOC
+}
+
 1;
